#ifndef AG_ARRAY_BASE_H
#define AG_ARRAY_BASE_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include <assert.h>
#include <stddef.h>
#include "agref.h"
#include "agcntnr.h"

template <class Object>
class AgArrayBase
: public AgReferenceCounter
{
  Object *data;
  int count;
public:
  AgArrayBase();
  AgArrayBase(int n);
  AgArrayBase(Object *x, int n);
  AgArrayBase(const AgArrayBase<Object> &x);
  AgArrayBase(const AgContainerContents<Object> &c);
  ~AgArrayBase();
  AgArrayBase<Object> &reset();
  AgArrayBase<Object> &resize(int);
  AgArrayBase<Object> &operator = (const AgArrayBase<Object> &x);
  AgArrayBase<Object> &operator = (const AgContainerContents<Object> &x);
  operator Object *() {return data;}
  operator const Object *() const {return data;}
  int size() const {return count;}
  Object &operator [] (int x) const;
};

template <class Object>
AgArrayBase<Object>::AgArrayBase()
: AgReferenceCounter(),
  data(NULL),
  count(0)
{
  // Nothing to do
}


template <class Object>
AgArrayBase<Object>::AgArrayBase(int n)
: AgReferenceCounter(),
  data(n ? new Object[n] : 0),
  count(n)
{
  // Nothing to do
}

template <class Object>
AgArrayBase<Object>::AgArrayBase(Object *x, int n)
: AgReferenceCounter(),
  data(new Object[n]),
  count(n)
{
  while (n--) data[n] = x[n];
}

template <class Object>
AgArrayBase<Object>::AgArrayBase(const AgArrayBase<Object> &x)
: AgReferenceCounter(),
  data(x.count ? new Object[x.count] : NULL),
  count(x.count)
{
  if (data == NULL) return;
  int n = count;
  while (n--) data[n] = x.data[n];
}

template <class Object>
AgArrayBase<Object>::AgArrayBase(const AgContainerContents<Object> &c)
: AgReferenceCounter(),
  data(c.size() ? new Object[c.size()] : NULL) ,
  count(c.size())
{
  if (data == NULL) return;
  int n = count;
  while (n--) data[n] = c[n];
}

template <class Object>
AgArrayBase<Object>::~AgArrayBase() {
  delete [] data;
}


template <class Object>
AgArrayBase<Object> &AgArrayBase<Object>::operator = (const AgArrayBase<Object> &x) {
  if (&x == this) return *this;
  delete [] data;
  count = x.count;
  if (count) {
    data = new Object[count];
    int n = count;
    while (n--) data[n] = x.data[n];
  }
  else data = NULL;
  return *this;
}


template <class Object>
AgArrayBase<Object> &AgArrayBase<Object>::operator = (const AgContainerContents<Object> &x) {
  delete [] data;
  count = x.size();
  if (count) {
    data = new Object[count];
    int n = count;
    while (n--) data[n] = x[n];
  }
  else data = NULL;
  return *this;
}

template <class Object>
AgArrayBase<Object> &AgArrayBase<Object>::reset() {
  delete [] data;
  count = 0;
  data = NULL;
  return *this;
}

template <class Object>
AgArrayBase<Object> &AgArrayBase<Object>::resize(int n) {
  int m = n;
  if (m > count) m = count;
  Object *newData = new Object[n];
  while (m--) newData[m] = data[m];
  delete [] data;
  data = newData;
  count = n;
  return *this;
}

template <class Object>
Object &AgArrayBase<Object>::operator [] (int x) const {
  assert((unsigned) x < (unsigned) count);
  return data[x];
}

template <class Object>
int operator == (const AgArrayBase<Object> &x, const AgArrayBase<Object> &y) {
  if (&x == &y) return 1;
  int n = x.size();
  if (n != y.size()) return 0;
  while (n--) if (x[n] != y[n]) return 0;
  return 1;
}

template <class Object>
inline int operator != (const AgArrayBase<Object> &x, const AgArrayBase<Object> &y) {
  return !(x == y);
}

#endif
